@@ -93,6 +93,9 @@ class TestCoveragePlugin(Plugin):
         # test test_name->dict(filename->set(linenos))
         self.test_data = defaultdict(dict)
 
+        # the root directory of our diff (this is basically cwd)
+        self.root = None
+
         report_output = config.report_output
         if not report_output or report_output == '-':
             self.report_file = None
@@ -166,6 +169,9 @@ class TestCoveragePlugin(Plugin):
 
             filename = filename[2:]
 
+            if self.root is None:
+                self.root = os.path.abspath(filename)[:-len(filename)]
+
             # Ignore non python files
             if not is_py_script(filename):
                 continue
@@ -203,6 +209,94 @@ class TestCoveragePlugin(Plugin):
 
             self.logger.info("Determined available coverage in %.2fs with %d test(s)", time.time() - s, len(pending_funcs))
 
+    def wantMethod(self, method):
+        if not self.config.discover:
+            return
+
+        # only works with unittest compatible functions currently
+        method_name = method.__name__
+        method = getattr(sys.modules[method.im_class.__module__], method.im_class.__name__)
+        test_name = '%s:%s.%s' % (method.__module__, method.__name__, method_name)
+
+        # test has coverage for diff
+        if test_name in self.pending_funcs:
+            return True
+
+        # check if this test was modified (e.g. added/changed)
+        filename = inspect.getfile(method)
+        if self.root and filename.startswith(self.root):
+            filename = filename[len(self.root):]
+        diff_data = self.diff_data[filename]
+        if diff_data:
+            lines, startlineno = inspect.getsourcelines(method)
+            for lineno in xrange(startlineno, len(lines) + startlineno):
+                if lineno in diff_data:
+                    self.pending_funcs.add(test_name)
+                    self.logger.info("Adding test due to new or changed code: %s", test_name)
+                    return True
+
+        # test has no coverage recorded, defer to other plugins
+        if self.config.test_missing and not self.db.has_test(self.revision_id, test_name):
+            self.pending_funcs.add(test_name)
+            self.logger.info("Allowing test due to missing coverage report: %s", test_name)
+            return None
+
+        return False
+
+    def startTest(self, test):
+        if not (self.config.record or self.config.report):
+            return
+
+        self.coverage.start()
+
+    def stopTest(self, test):
+        if not (self.config.record or self.config.report):
+            return
+
+        cov = self.coverage
+        cov.stop()
+
+        # this must have been imported under a different name
+        # if self.discover and test_name not in self.pending_funcs:
+        #     self.logger.warning("Unable to determine origin for test: %s", test_name)
+        #     return
+
+        # initialize reporter
+        rep = Reporter(cov)
+
+        # process all files
+        rep.find_code_units(None, cov.config)
+
+        # Compute the standard deviation for all code executed from this test
+        linenos = []
+        for filename in cov.data.measured_files():
+            linenos.extend(cov.data.executed_lines(filename).values())
+
+        # We're recording so fetch the test data
+        if self.config.record:
+            test_ = test.test
+            test_name = self._get_name_from_test(test_)
+            test_data = self.test_data[test_name]
+
+        for cu in rep.code_units:
+            # if sys.modules[test_.__module__].__file__ == cu.filename:
+            #     continue
+            filename = cu.name + '.py'
+            linenos = cov.data.executed_lines(cu.filename)
+
+            if self.config.record:
+                linenos_in_prox = dict((k, v) for k, v in linenos.iteritems() if v < self.config.max_distance)
+                if linenos_in_prox:
+                    test_data[filename] = linenos_in_prox
+
+            if self.config.report:
+                diff = self.diff_data[filename]
+                cov_linenos = [l for l in linenos if l in diff]
+                if cov_linenos:
+                    self.cov_data[filename].update(cov_linenos)
+
+        cov.erase()
+
     def report(self, stream):
         if self.config.record:
             self._record_test_coverage()
@@ -273,89 +367,3 @@ class TestCoveragePlugin(Plugin):
                     if not linenos:
                         continue
                     stream.writeln('%-35s   %s' % (filename, ', '.join(map(str, sorted(linenos)))))
-
-    def wantMethod(self, method):
-        if not self.config.discover:
-            return
-
-        # only works with unittest compatible functions currently
-        method_name = method.__name__
-        method = getattr(sys.modules[method.im_class.__module__], method.im_class.__name__)
-        test_name = '%s:%s.%s' % (method.__module__, method.__name__, method_name)
-
-        # test has coverage for diff
-        if test_name in self.pending_funcs:
-            return True
-
-        # check if this test was modified (e.g. added/changed)
-        diff_data = self.diff_data[inspect.getfile(method)]
-        if diff_data:
-            lines, startlineno = inspect.getsourcelines(method)
-            for lineno in xrange(startlineno, len(lines) + startlineno):
-                if lineno in diff_data:
-                    self.pending_funcs.add(test_name)
-                    self.logger.info("Adding test due to new or changed code: %s", test_name)
-                    return True
-
-        # test has no coverage recorded, defer to other plugins
-        if self.config.test_missing and not self.db.has_test(self.revision_id, test_name):
-            self.pending_funcs.add(test_name)
-            self.logger.info("Allowing test due to missing coverage report: %s", test_name)
-            return None
-
-        return False
-
-    def startTest(self, test):
-        if not (self.config.record or self.config.report):
-            return
-
-        self.coverage.start()
-
-    def stopTest(self, test):
-        if not (self.config.record or self.config.report):
-            return
-
-        cov = self.coverage
-        cov.stop()
-
-        # this must have been imported under a different name
-        # if self.discover and test_name not in self.pending_funcs:
-        #     self.logger.warning("Unable to determine origin for test: %s", test_name)
-        #     return
-
-        # initialize reporter
-        rep = Reporter(cov)
-
-        # process all files
-        rep.find_code_units(None, cov.config)
-
-        # Compute the standard deviation for all code executed from this test
-        linenos = []
-        for filename in cov.data.measured_files():
-            linenos.extend(cov.data.executed_lines(filename).values())
-
-        # We're recording so fetch the test data
-        if self.config.record:
-            test_ = test.test
-            test_name = self._get_name_from_test(test_)
-            test_data = self.test_data[test_name]
-
-        for cu in rep.code_units:
-            # if sys.modules[test_.__module__].__file__ == cu.filename:
-            #     continue
-            filename = cu.name + '.py'
-            linenos = cov.data.executed_lines(cu.filename)
-
-            if self.config.record:
-                linenos_in_prox = dict((k, v) for k, v in linenos.iteritems() if v < self.config.max_distance)
-                if linenos_in_prox:
-                    test_data[filename] = linenos_in_prox
-
-            if self.config.report:
-                diff = self.diff_data[filename]
-                cov_linenos = [l for l in linenos if l in diff]
-                if cov_linenos:
-                    self.cov_data[filename].update(cov_linenos)
-
-        cov.erase()
-
